# -*- encoding: utf-8 -*-
#オリジナルライセンス
require 'spec_helper'

describe OriginalLicense do
  before do
    #テストデータを用意してね
    @f = Rails.root + 'spec/json/original_license.json'
    @t = File.open(@f, 'r').read
    @j = JSON.parse @t
    @fs = Rails.root + 'spec/json/original_licenses.json'
    @ts = File.open(@fs, 'r').read
    @js = JSON.parse @ts
    @fes = Rails.root + 'spec/json/invalid_original_licenses.json'
    @tes = File.open(@fes, 'r').read
    @jes = JSON.parse @tes
  end
  describe '検証に於いて' do
    before do
      @l = Factory :license
    end
    
    it 'オーソドックスなデータなら通る' do
      @ol = Factory.build :original_license, :license_id => @l.id
      @ol.should be_valid
    end
    
    context 'nameを検証するとき' do
      before do
        @ol = Factory.build :original_license, :license_id => @l.id
      end
      it 'テストデータの確認' do
        @ol.name = 'CC by'
        @ol.should be_valid
      end
      it 'nullなら失敗する' do
        @ol.name = ''
        @ol.should_not be_valid
      end
      it '51文字以上なら失敗する' do
        @ol.name = 'a'*51
        @ol.should_not be_valid
      end
    end
    context 'urlを検証するとき' do
      before do
        @ol = Factory.build :original_license, :license_id => @l.id
      end
      it 'テストデータの確認' do
        @ol.url = 'CC by'
        @ol.should be_valid
      end
      it 'nullなら失敗する' do
        @ol.url = ''
        @ol.should_not be_valid
      end
      it '201文字以上なら失敗する' do
        @ol.url = 'a'*201
        @ol.should_not be_valid
      end
      it '重複していたら失敗する' do
        cl = Factory :original_license, :license_id => @l.id
        @ol.should_not be_valid
      end
      it 'url形式でなら失敗する' do
        @ol.url = ''
        pending
      end
    end
    context 'license_idを検証するとき' do
      before do
        @ol = Factory.build :original_license
      end
      it 'テストデータの確認' do
        @ol.license_id = @l.id
        @ol.should be_valid
      end
      it 'nullなら失敗する' do
        @ol.license_id = nil
        @ol.should_not be_valid
      end
      it '数値でなければ失敗する' do
        @ol.license_id = 'a'
        @ol.should_not be_valid
      end
      it '存在するlicenseでなければ失敗する' do
        @ol.license_id = 0
        @ol.should_not be_valid
      end
    end
  end
  
  describe '対象オリジナルライセンスの取得に於いて' do
    before do
      @lc = Factory :license
      @ol = Factory :original_license, :license_id => @lc.id
    end
    context 'urlが一致するオリジナルライセンスがないとき' do
      it '新規オリジナルライセンスを準備して返す' do
        r = OriginalLicense.update_original_license Factory.attributes_for(:original_license, :url => 'http://domain.no')
        r.should be_a_new OriginalLicense
      end
    end
    context 'urlが一致するオリジナルライセンスがあるとき' do
      it '該当オリジナルライセンスを返す' do
        prm = Factory.attributes_for(:original_license)
        r = OriginalLicense.update_original_license prm
        r.is_a?(OriginalLicense).should be_true
        r.should_not be_a_new OriginalLicense
        r[:url].should eq prm[:url]
      end
    end
  end
  
  describe 'オリジナルライセンス更新に於いて' do
    before do
      @lc = Factory :license
      @ol = Factory :original_license, :license_id => @lc.id, :url => @lc.url
      @attr = Factory.attributes_for(:original_license, :name => 'exist case', :url => @lc.url)
      @newattr = Factory.attributes_for(:original_license, :url => 'http://domain.no')
    end
    context 'つつがなく終わるとき' do
      it '対象オリジナルライセンスを問い合わせている' do
        OriginalLicense.stub(:update_original_license).with(any_args).and_return(OriginalLicense.new(@attr))
        OriginalLicense.should_receive(:update_original_license).exactly(1)
        OriginalLicense.store @attr
      end
      context '新規のとき' do
        it 'オリジナルライセンスを保存しようとしている' do
          OriginalLicense.any_instance.should_receive(:save).exactly(1)
          OriginalLicense.store @newattr
        end
        it 'オリジナルライセンスが作成されている' do
          lambda {
            OriginalLicense.store @newattr
          }.should change OriginalLicense, :count
        end
        context 'ライセンスとの連動' do
          it 'ライセンスが作成されている' do
            lambda {
              OriginalLicense.store @newattr
            }.should change License, :count
          end
          it '両者がリンクされている' do
            r = OriginalLicense.store @newattr
            l = License.find(r.license_id)
            l.should be_true
          end
          it '属性が一致している' do
            r = OriginalLicense.store @newattr
            l = License.find(r.license_id)
            l.url.should eq r.url
            l.name.should eq r.name
          end
        end
      end
      context '更新のとき' do
        it 'オリジナルライセンスを保存しようとしている' do
          OriginalLicense.any_instance.should_receive(:save).exactly(1)
          OriginalLicense.store @attr
        end
        it 'オリジナルライセンスの数に変化がない' do
          lambda {
            OriginalLicense.store @attr
          }.should_not change OriginalLicense, :count
        end
        context 'ライセンスとの連動' do
          it 'ライセンスの数に変化がない' do
            lambda {
              OriginalLicense.store @attr
            }.should_not change License, :count
          end
          it '両者がリンクされている' do
            r = OriginalLicense.store @attr
            l = License.find(r.license_id)
            l.should be_true
          end
          it '属性が一致している' do
            r = OriginalLicense.store @attr
            l = License.find(r.license_id)
            l.url.should eq r.url
            l.name.should eq r.name
          end
        end
      end
      it '属性が一致している' do
        r = OriginalLicense.store @newattr
        r.url.should eq @newattr[:url]
        r.name.should eq @newattr[:name]
      end
      it '保存されたOriginalLicenseオブジェクトを返す' do
        r = OriginalLicense.store @newattr
        r.should_not be_a_new OriginalLicense
      end
    end
    context 'ライセンスの作成に失敗するとき' do
      before do
        License.any_instance.stub(:save).with(any_args).and_return(false)
      end
      context '新規のとき' do
        it 'ライセンスに変化がない' do
          lambda {
            r = OriginalLicense.store @newattr
          }.should_not change License, :count
        end
        it 'オリジナルライセンスに変化がない' do
          lambda {
            r = OriginalLicense.store @newattr
          }.should_not change OriginalLicense, :count
        end
      end
      context '更新のとき' do
        it 'オリジナルライセンス属性に変化がない' do
          lambda {
            r = OriginalLicense.store @attr
          }.should_not change License.find(@ol.id), :name
        end
        it 'ライセンス属性に変化がない' do
          lambda {
            r = OriginalLicense.store @attr
          }.should_not change License.find(@lc.id), :name
        end
      end
    end
    context 'オリジナルライセンスの作成に失敗するとき' do
      before do
        OriginalLicense.any_instance.stub(:save).with(any_args).and_return(false)
      end
      context '新規のとき' do
        it 'ライセンスに変化がない' do
          lambda {
            r = OriginalLicense.store @newattr
          }.should_not change License, :count
        end
        it 'オリジナルライセンスに変化がない' do
          lambda {
            r = OriginalLicense.store @newattr
          }.should_not change OriginalLicense, :count
        end
      end
      context '更新のとき' do
        it 'オリジナルライセンス属性に変化がない' do
          lambda {
            r = OriginalLicense.store @attr
          }.should_not change License.find(@ol.id), :name
        end
        it 'ライセンス属性に変化がない' do
          lambda {
            r = OriginalLicense.store @attr
          }.should_not change License.find(@lc.id), :name
        end
      end
    end
  end
  
  describe '単体取得に於いて' do
    before do
      @ol = Factory.build :original_license
      @ol.store
    end
    it '指定のライセンスを返す' do
      l = OriginalLicense.show @ol.id
      l.should eq @ol
    end
    context '関連テーブルオプションがないとき' do
      it 'ライセンスだけを含んでいる' do
        r = OriginalLicense.show_include_opt
        r.should eq [:license]
      end
    end
    context '関連テーブルオプションで作家を含ませたとき' do
      it 'ライセンスと作家を含んでいる' do
        r = OriginalLicense.show_include_opt(:include => :author)
        r.should eq [:license, :author]
      end
    end
  end
  describe '一覧取得に於いて' do
    before do
      @ol = Factory.build :original_license
      @ol.store
    end
    context 'page補正について' do
      it '文字列から数値に変換される' do
        OriginalLicense.page('8').should eq 8
      end
      it 'nilの場合は1になる' do
        OriginalLicense.page().should eq 1
      end
      it '0以下の場合は1になる' do
        OriginalLicense.page('0').should eq 1
      end
    end
    context 'page_size補正について' do
      it '文字列から数値に変換される' do
        OriginalLicense.page_size('7').should eq 7
      end
      it 'nilの場合はOriginalLicense.default_page_sizeになる' do
        OriginalLicense.page_size().should eq OriginalLicense.default_page_size
      end
      it '0以下の場合はOriginalLicense.default_page_sizeになる' do
        OriginalLicense.page_size('0').should eq OriginalLicense.default_page_size
      end
      it 'OriginalLicense.max_page_sizeを超えた場合はOriginalLicense.max_page_sizeになる' do
        OriginalLicense.page_size('1000').should eq OriginalLicense.max_page_size
      end
    end
    it 'リストを返す' do
      l = OriginalLicense.list
      l.should eq [@ol]
    end
    it '名前順で並んでいる' do
      n = Factory.build :original_license, :url => 'http://tes.to', :name => 'peta2.2'
      n.store
      l = OriginalLicense.list
      l.should eq [@ol, n]
    end
    context 'DBに5件あって1ページの件数を2件に変えたとして' do
      before do
        @license2 = Factory.build :original_license, :url => 'http://tes.to2', :name => 'peta2.2'
        @license2.store
        @license3 = Factory.build :original_license, :url => 'http://tes.to3', :name => 'peta2.3'
        @license3.store
        @license4 = Factory.build :original_license, :url => 'http://tes.to4', :name => 'peta2.4'
        @license4.store
        @license5 = Factory.build :original_license, :url => 'http://tes.to5', :name => 'peta2.5'
        @license5.store
        OriginalLicense.stub(:default_page_size).and_return(2)
      end
      it '通常は2件を返す' do
        l = OriginalLicense.list
        l.should have(2).items 
      end
      it 'page=1なら末尾2件を返す' do
        #時系列で並んでいる
        l = OriginalLicense.list({}, 1)
        l.should eq [@ol, @license2]
      end
      it 'page=2なら中間2件を返す' do
        l = OriginalLicense.list({}, 2)
        l.should eq [@license3, @license4]
      end
      it 'page=3なら先頭1件を返す' do
        l = OriginalLicense.list({}, 3)
        l.should eq [@license5]
      end
    end
  end
  
  describe 'Json解析に於いて' do
    before do
    end
    context 'テキストを渡されたとき' do
      it 'Json解析している' do
        JSON.should_receive(:parse).with(@t).exactly(1)
        r = OriginalLicense.parse @t
      end
      it '単数データならHashで返す' do
        r = OriginalLicense.parse @t
        r.is_a?(Hash).should be_true
      end
      it '複数データならArrayで返す' do
        r = OriginalLicense.parse @ts
        r.is_a?(Array).should be_true
      end
    end
    context 'パース失敗したとき' do
      it 'Falseを返す' do
        JSON.should_receive(:parse).with(any_args).and_raise('StandardError')
        r = OriginalLicense.parse @t
        r.should be_false
      end
    end
  end
  
  describe 'Jsonの繰り返し処理に於いて' do
    before do
    end
    context '単体データを渡されたとき' do
      it '一回処理' do
        r = []
        OriginalLicense.each_license @j do |i|
          r << i
        end
        r.size.should eq 1
      end
    end
    context '複数を渡されたとき' do
      it '二回処理' do
        r = []
        OriginalLicense.each_license @js do |i|
          r << i
        end
        r.size.should eq 2
      end
    end
  end
  
  describe 'テキスト取り込みに於いて' do
    #成功でTrue、パース失敗でFalse、失敗は保存エラーのモデルを配列で返す
    #Licenseとの連動が完成していること
    before do
    end
    context 'つつがなく終わるとき' do
      it 'Json解析を依頼する' do
        OriginalLicense.should_receive(:parse).with(any_args).exactly(1)
        OriginalLicense.stub(:parse).with(any_args).and_return(@j)
        OriginalLicense.import(@t)
      end
      it '繰り返し処理を依頼する' do
        OriginalLicense.should_receive(:each_license).with(any_args).exactly(1)
        OriginalLicense.import(@t)
      end
      it 'ライセンス更新を一回依頼する' do
        OriginalLicense.stub(:store).with(any_args).and_return(OriginalLicense.new)
        OriginalLicense.any_instance.stub(:valid?).with(any_args).and_return(true)
        OriginalLicense.should_receive(:store).with(any_args).exactly(1)
        OriginalLicense.import(@t)
      end
      it 'オリジナルライセンスが追加される' do
        lambda {
          OriginalLicense.import(@t)
        }.should change OriginalLicense, :count
      end
      it '[]を返す' do
        OriginalLicense.import(@t).should eq []
      end
    end
    context '複数データがつつがなく終わるとき' do
      it 'ライセンス更新を二回依頼する' do
        OriginalLicense.stub(:store).with(any_args).and_return(OriginalLicense.new)
        OriginalLicense.any_instance.stub(:valid?).with(any_args).and_return(true)
        OriginalLicense.should_receive(:store).with(any_args).exactly(2)
        OriginalLicense.import(@ts)
      end
      it 'オリジナルライセンスが二個追加される' do
        lambda {
          OriginalLicense.import(@ts)
        }.should change(OriginalLicense, :count).by 2
      end
      it '[]を返す' do
        OriginalLicense.import(@ts).should eq []
      end
    end
    #例外ケース
    context 'Json解析に失敗したとき' do
      before do
        OriginalLicense.stub(:parse).with(any_args).and_return(false)
      end
      it 'オリジナルライセンスの数に変化がない' do
        lambda {
          OriginalLicense.import(@t)
        }.should_not change OriginalLicense, :count
      end
      it 'Falseを返す' do
        OriginalLicense.import(@t).should be_false
      end
    end
    context 'オリジナルライセンス作成に失敗したとき' do
      before do
        OriginalLicense.any_instance.stub(:save).with(any_args).and_return(false)
        OriginalLicense.any_instance.stub(:valid?).with(any_args).and_return(false)
      end
      it 'オリジナルライセンスの数に変化がない' do
        lambda {
          OriginalLicense.import(@t)
        }.should_not change OriginalLicense, :count
      end
      it '配列を返す' do
        r = OriginalLicense.import(@t)
        r.is_a?(Array).should be_true
      end
      it '配列の中身は一件' do
        r = OriginalLicense.import(@t)
        r.should have(1).items
      end
      it 'オリジナルライセンスオブジェクトが入っている' do
        r = OriginalLicense.import(@t)
        r.first.is_a?(OriginalLicense).should be_true
      end
    end
    context '複数のオリジナルライセンス作成に失敗したとき' do
      #三件中、二件の失敗、一件を成功させ、成功データは戻り値に含まないことを確認する
      it 'オリジナルライセンスの数に変化がない' do
        lambda {
          OriginalLicense.import(@tes)
        }.should_not change OriginalLicense, :count
      end
      it '途中で保存に失敗しても全件更新依頼する' do
        OriginalLicense.stub(:store).with(any_args).and_return(OriginalLicense.new)
        OriginalLicense.should_receive(:store).with(any_args).exactly(3)
        OriginalLicense.import(@tes)
      end
      it '配列を返す' do
        r = OriginalLicense.import(@tes)
        r.is_a?(Array).should be_true
      end
      it '配列の中身は2件' do
        r = OriginalLicense.import(@tes)
        r.should have(2).items
      end
      it '配列の中身は失敗したオリジナルライセンスオブジェクトが入っている' do
        r = OriginalLicense.import(@tes)
        r[0].is_a?(OriginalLicense).should be_true
        r[0]["name"].should eq 'fail1'
        r[1].is_a?(OriginalLicense).should be_true
        r[1]["name"].should eq 'fail2'
      end
    end
  end
  
  describe 'ファイル取り込みに於いて' do
    before do
      OriginalLicense.stub(:import).with(any_args).and_return(true)
    end
    context 'つつがなく終わるとき' do
      before do
        OriginalLicense.stub(:import).with(any_args).and_return(true)
      end
      it 'ファイルを開いてテキストを読む' do
        File.should_receive(:open).with(@f, 'r').exactly(1)
        OriginalLicense.import_file(@f)
      end
      it 'テキスト取り込みを依頼する' do
        OriginalLicense.should_receive(:import).with(any_args).exactly(1)
        OriginalLicense.import_file(@f)
      end
      #テキスト取り込み成功でTrueが返る
      it 'Trueを返す' do
        OriginalLicense.import_file(@f).should be_true
      end
    end
    context 'ファイルが開けないとき' do
      before do
        File.stub(:open).with(any_args).and_raise('StandardError')
      end
      it 'ファイルエラーのメッセージを出力する' do
        pending
      end
      it 'Falseを返す' do
        OriginalLicense.import_file(@f).should be_false
      end
    end
    #失敗したときは、失敗したライセンスが配列で返る
    context 'テキスト取り込みが失敗したとき' do
      before do
        OriginalLicense.stub(:import).with(any_args).and_return(false)
      end
      it '各オリジナルライセンスのエラーメッセージを出力する' do
        pending
      end
      it 'Falseを返す' do
        OriginalLicense.import_file(@f).should be_false
      end
    end
  end
  
end
